@@ -1,12 +1,12 @@
 ;;; corfu-popupinfo.el --- Candidate information popup for Corfu -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
 
 ;; Author: Yuwei Tian <fishtai0@gmail.com>, Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 1.1
-;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4") (corfu "1.1"))
+;; Version: 1.2
+;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4") (corfu "1.2"))
 ;; Homepage: https://github.com/minad/corfu
 
 ;; This file is part of GNU Emacs.
@@ -111,7 +111,6 @@ documentation from the backend is usually expensive."
            (const vertical)
            (const force-left)
            (const force-right)
-           (const force-horizontal)
            (const force-vertical)))
   :group 'corfu)
 
@@ -134,38 +133,40 @@ documentation from the backend is usually expensive."
     (fringe-indicator-alist (continuation)))
   "Buffer parameters.")
 
-(defvar-local corfu-popupinfo--toggle 'init
-  "Local toggle state.")
-
-(defvar-local corfu-popupinfo--function
-  #'corfu-popupinfo--get-documentation
-  "Function called to obtain documentation string.")
-
 (defvar corfu-popupinfo--frame nil
   "Info popup child frame.")
 
 (defvar corfu-popupinfo--timer nil
   "Corfu info popup auto display timer.")
 
-(defvar-local corfu-popupinfo--candidate nil
+(defvar corfu-popupinfo--toggle 'init
+  "Toggle state.")
+
+(defvar corfu-popupinfo--function
+  #'corfu-popupinfo--get-documentation
+  "Function called to obtain documentation string.")
+
+(defvar corfu-popupinfo--candidate nil
   "Completion candidate for the info popup.")
 
-(defvar-local corfu-popupinfo--coordinates nil
+(defvar corfu-popupinfo--coordinates nil
   "Coordinates of the candidate popup.
 The coordinates list has the form (LEFT TOP RIGHT BOTTOM) where
 all values are in pixels relative to the origin.  See
 `frame-edges' for details.")
 
-(defvar-local corfu-popupinfo--lock-dir nil
+(defvar corfu-popupinfo--lock-dir nil
   "Locked position direction of the info popup.")
 
-(defconst corfu-popupinfo--state-vars
-  '(corfu-popupinfo--candidate
-    corfu-popupinfo--coordinates
-    corfu-popupinfo--lock-dir
-    corfu-popupinfo--toggle
-    corfu-popupinfo--function)
-  "Buffer-local state variables used by corfu-popupinfo.")
+(defconst corfu-popupinfo--initial-state
+  (mapcar
+   (lambda (k) (cons k (symbol-value k)))
+   '(corfu-popupinfo--candidate
+     corfu-popupinfo--coordinates
+     corfu-popupinfo--lock-dir
+     corfu-popupinfo--toggle
+     corfu-popupinfo--function))
+  "Initial state of `corfu-popupinfo-mode'.")
 
 (defun corfu-popupinfo--visible-p (&optional frame)
   "Return non-nil if FRAME is visible."
@@ -178,7 +179,8 @@ all values are in pixels relative to the origin.  See
     (let ((old-buffers (buffer-list)) (buffer nil))
       (unwind-protect
           (when-let
-              ((fun (plist-get corfu--extra :company-location))
+              ((extra (nth 4 completion-in-region--data))
+               (fun (plist-get extra :company-location))
                ;; BUG: company-location may throw errors if location is not found
                (loc (ignore-errors (funcall fun candidate)))
                ((setq buffer
@@ -213,7 +215,8 @@ all values are in pixels relative to the origin.  See
 
 (defun corfu-popupinfo--get-documentation (candidate)
   "Get the documentation for CANDIDATE."
-  (when-let ((fun (plist-get corfu--extra :company-doc-buffer))
+  (when-let ((extra (nth 4 completion-in-region--data))
+             (fun (plist-get extra :company-doc-buffer))
              (res (save-excursion
                     (let ((inhibit-message t)
                           (message-log-max nil)
@@ -289,7 +292,7 @@ form (X Y WIDTH HEIGHT DIR)."
   (pcase-let*
       ((cw (default-font-width))
        (lh (default-line-height))
-       (border (alist-get 'child-frame-border-width corfu--frame-parameters))
+       (border (alist-get 'internal-border-width corfu--frame-parameters))
        (`(,_pfx ,_pfy ,pfw ,pfh)
         (corfu-popupinfo--frame-geometry (frame-parent corfu--frame)))
        (`(,cfx ,cfy ,cfw ,cfh) (corfu-popupinfo--frame-geometry corfu--frame))
@@ -359,7 +362,7 @@ form (X Y WIDTH HEIGHT DIR)."
           (corfu-popupinfo--hide)
           (setq cand-changed nil coords-changed nil)))
       (when (or cand-changed coords-changed)
-        (pcase-let* ((border (alist-get 'child-frame-border-width corfu--frame-parameters))
+        (pcase-let* ((border (alist-get 'internal-border-width corfu--frame-parameters))
                      (`(,area-x ,area-y ,area-w ,area-h ,area-d)
                       (corfu-popupinfo--area
                        (if cand-changed
@@ -504,12 +507,10 @@ not be displayed until this command is called again, even if
               (corfu-popupinfo--hide))))
       (corfu-popupinfo--hide))))
 
-(cl-defmethod corfu--teardown :before (&context (corfu-popupinfo-mode (eql t)))
+(cl-defmethod corfu--teardown :before (_buf &context (corfu-popupinfo-mode (eql t)))
   (corfu-popupinfo--hide)
-  (mapc #'kill-local-variable corfu-popupinfo--state-vars)
-  (setq minor-mode-overriding-map-alist
-        (assq-delete-all #'corfu-popupinfo-mode
-                         minor-mode-overriding-map-alist)))
+  (cl-loop for (k . v) in corfu-popupinfo--initial-state do (set k v))
+  (cl-callf2 assq-delete-all #'corfu-popupinfo-mode minor-mode-overriding-map-alist))
 
 ;; Emacs 28: Do not show Corfu commands with M-X
 (dolist (sym '(corfu-popupinfo-scroll-down corfu-popupinfo-scroll-up
